const vm = require('vm')

class VM {
  /**
   * Creates a new VM with the set script.
   * @param {String} [script=null]  Script to compile
   * @param {Object} [options={}]   Script options
   * @param {Object} [sandbox=null] Sandbox object
   */
  constructor (script = null, options = {}, sandbox = {}) {
    this._script = new vm.Script(script, options)
    this._sandbox = sandbox
    this._handler = null
  }

  /**
   * Runs the script and sets the handler.
   * @return {Promise} Resolves if the handler was successfully set
   */
  run () {
    return new Promise((resolve, reject) => {
      const context = new vm.createContext(this._sandbox)
      this._script.runInContext(context)
      process.nextTick(() => {
        if (context.process && typeof context.process === 'function') {
          this._handler = context.process
          return resolve()
        } else {
          return reject(new TypeError('process must be a function'))
        }
      })
    })
  }

  /**
   * Processes the handler with the given row.
   * @param {Object} [row={}] Row object to use
   * @return {Boolean} A boolean retrieved from the process method
   */
  process (row = {}) {
    if (typeof this._handler !== 'function') {
      throw new Error('handler not set')
    }
    return this._handler(row)
  }
}

exports = module.exports = VM
